@@ -1,9 +1,9 @@
-# $Id: V2_0.pm 33 2007-03-14 03:06:58Z daisuke $
+# $Id: V0_91.pm 36 2007-03-23 05:25:29Z daisuke $
 #
 # Copyright (c) 2005-2007 Daisuke Maki <daisuke@endeworks.jp>
 # All rights reserved.
 
-package XML::RSS::LibXML::V2_0;
+package XML::RSS::LibXML::V0_91;
 use strict;
 use warnings;
 use base qw(XML::RSS::LibXML::ImplBase);
@@ -13,12 +13,13 @@ use DateTime::Format::Mail;
 my %DcElements = (
     (map { ("dc:$_" => [ { module => 'dc', element => $_ } ]) }
         qw(language rights date publisher creator title subject description contributer type format identifier source relation coverage)),
-);
-    
+);  
+            
 my %SynElements = (
     (map { ("syn:$_" => [ { module => 'syn', element => $_ } ]) }
         qw(updateBase updateFrequency updatePeriod)),
-);
+);  
+
 my $format_dates = sub {
     my $v = eval {
         DateTime::Format::Mail->format_datetime(
@@ -50,51 +51,43 @@ my %ChannelElements = (
     category => [ { module => 'dc', element => 'category' }, 'category' ],
     generator => [ { module => 'dc', element => 'generator' }, 'generator' ],
     ttl => [ { module => 'dc', element => 'ttl' }, 'ttl' ],
-    image => [ 'image' ],
-); 
+    rating => [ 'rating' ],
+);
+delete $ChannelElements{'dc:creator'};
 
 my %ItemElements = (
     %DcElements,
-    enclosure => ['enclosure'],
-    guid      => ['guid'],
     map { ($_ => [$_]) }
         qw(title link description author category comments pubDate)
 );
 
 my %ImageElements = (
-    (map { ($_ => [$_]) } qw(title url link width height description)),
+    (map { ($_ => [$_]) } qw(title url link description width height)),
     %DcElements,
-);      
-        
+);
+
 my %TextInputElements = (
     (map { ($_ => [$_]) } qw(title link description name)),
     %DcElements
 );
 
-sub definition
+sub definition 
 {
     return +{
         channel => {
             title          => '',
-            'link'         => '',
-            description    => '',
-            language       => undef,
             copyright      => undef,
-            managingEditor => undef,
-            webMaster      => undef,
-            pubDate        => undef,
-            lastBuildDate  => undef,
-            category       => undef,
-            generator      => undef,
+            description    => '',
             docs           => undef,
-            cloud          => '',
-            ttl            => undef,
-            image          => '',
-            textinput      => '',
-            skipHours      => '',
-            skipDays       => '',
+            language       => undef,
+            lastBuildDate  => undef,
+            'link'         => '',
+            managingEditor => undef,
+            pubDate        => undef,
+            rating         => undef,
+            webMaster      => undef,
         },
-        image => bless ({
+        image => bless({
             title       => undef,
             url         => undef,
             'link'      => undef,
@@ -102,19 +95,55 @@ sub definition
             height      => undef,
             description => undef,
         }, 'XML::RSS::LibXML::ElementSpec'),
-        skipDays  => bless ({
-            day => undef,
-        }, 'XML::RSS::LibXML::ElementSpec'),
-        skipHours => bless ({
-            hour => undef,
-        }, 'XML::RSS::LibXML::ElementSpec'),
-        textinput => bless ({
+        skipDays  => {day  => undef,},
+        skipHours => {hour => undef,},
+        textinput => bless({
             title       => undef,
             description => undef,
             name        => undef,
             'link'      => undef,
         }, 'XML::RSS::LibXML::ElementSpec'),
-    };
+    }
+}
+
+sub accessor_definition
+{
+    return +{
+        channel => {
+            "title"          => [1, 100],
+            "description"    => [1, 500],
+            "link"           => [1, 500],
+            "language"       => [1, 5],
+            "rating"         => [0, 500],
+            "copyright"      => [0, 100],
+            "pubDate"        => [0, 100],
+            "lastBuildDate"  => [0, 100],
+            "docs"           => [0, 500],
+            "managingEditor" => [0, 100],
+            "webMaster"      => [0, 100],
+        },
+        image => {
+            "title"       => [1, 100],
+            "url"         => [1, 500],
+            "link"        => [0, 500],
+            "width"       => [0, 144],
+            "height"      => [0, 400],
+            "description" => [0, 500]
+        },
+        item => {
+            "title"       => [1, 100],
+            "link"        => [1, 500],
+            "description" => [0, 500]
+        },
+        textinput => {    
+            "title"       => [1, 100],
+            "description" => [1, 500],
+            "name"        => [1, 20],
+            "link"        => [1, 500]
+        },
+        skipHours => {"hour" => [1, 23]},
+        skipDays  => {"day"  => [1, 10]}
+    }
 }
 
 sub parse_dom
@@ -124,11 +153,10 @@ sub parse_dom
     my $dom  = shift;
 
     $c->reset;
-    $c->version('2.0');
+    $c->version('0.91');
     $self->parse_namespaces($c, $dom);
     $self->parse_channel($c, $dom);
     $self->parse_items($c, $dom);
-    $self->parse_misc_simple($c, $dom);
 }
 
 sub parse_channel
@@ -138,21 +166,25 @@ sub parse_channel
     my $xc = $c->create_xpath_context($c->{namespaces});
 
     my ($root) = $xc->findnodes('/rss/channel', $dom);
-    my %h = $self->parse_children($c, $root, './*[name() != "item"]');
+    my %h = $self->parse_children($c, $root);
 
+    foreach my $type qw(day hour) {
+        my $field = 'skip' . ucfirst($type) . 's';
+        if (my $skip = delete $h{$field}) {
+            $c->$field(%$skip);
+        }
+    }
     $c->channel(%h);
-    if ($h{textinput}) {
-        $c->{textinput} = $h{textinput};
-    }
-    if ($h{image}) {
-        $c->{image} = $h{image};
-    }
 }
 
 sub parse_items
 {
-    my ($self, $c, $dom) = @_;
+    my $self    = shift;
+    my $c       = shift;
+    my $dom     = shift;
+
     my @items;
+
     my $version = $c->version;
     my $xc      = $c->create_xpath_context($c->{namespaces});
     my $xpath   = '/rss/channel/item';
@@ -162,63 +194,26 @@ sub parse_items
     }
 }
 
-sub parse_misc_simple
+sub create_dtd
 {
-    my ($self, $c, $dom) = @_;
+    my $self = shift;
+    my $c    = shift;
+    my $dom  = shift;
 
-    my $xc = $c->create_xpath_context($c->{namespaces});
-    foreach my $node ($xc->findnodes('/rss/*[name() != "channel" and name() != "item"]', $dom)) {
-        my $h = $self->parse_children($c, $node);
-        my $name = $node->localname;
-        my $prefix = $node->getPrefix();
-
-        $name = 'textinput' if $name eq 'textInput';
-
-        if ($prefix) {
-            $c->{$prefix} ||= {};
-            $self->store_element($c->{$prefix}, $name, $h);
-
-            # XML::RSS requires us to allow access to elements both from
-            # the prefix and the namespace
-            $c->{$c->{namespaces}{$prefix}} ||= {};
-            $self->store_element($c->{$c->{namespaces}{$prefix}}, $name, $h);
-        } else {
-            $self->store_element($c, $name, $h);
-        }
-    }
-}
-
-sub create_dom
-{
-    my ($self, $c) = @_;
-
-    my $dom = $self->SUPER::create_dom($c);
-    my $root = $dom->getDocumentElement();
-    my $xc = $c->create_xpath_context(scalar $c->namespaces);
-    my($channel) = $xc->findnodes('/rss/channel', $dom);
-
-    if (my $image = $c->image) {
-        my $inode = $dom->createElement('image');
-        $self->create_element_from_spec($image, $dom, $inode, \%ImageElements);
-        $self->create_extra_modules($image, $dom, $inode, $c->namespaces);
-        $root->appendChild($inode);
-    }
-
-    if (my $textinput = $c->textinput) {
-        my $inode = $dom->createElement('textInput');
-        $self->create_element_from_spec($textinput, $dom, $inode, \%TextInputElements);
-        $self->create_extra_modules($textinput, $dom, $inode, $c->namespaces);
-        $root->appendChild($inode);
-    }
-
-    return $dom;
+    my $dtd = $dom->createExternalSubset(
+        'rss',
+        '-//Netscape Communications//DTD RSS 0.91//EN',
+        'http://my.netscape.com/publish/formats/rss-0.91.dtd'
+    );
+    $dom->setInternalSubset($dtd);
 }
 
 sub create_rootelement
 {
     my ($self, $c, $dom) = @_;
+
     my $root = $dom->createElement('rss');
-    $root->setAttribute(version => '2.0');
+    $root->setAttribute(version => '0.91');
     $dom->setDocumentElement($root);
 }
 
@@ -228,8 +223,41 @@ sub create_channel
 
     my $root = $dom->getDocumentElement();
     my $channel = $dom->createElement('channel');
-
     $self->create_element_from_spec($c->channel, $dom, $channel, \%ChannelElements);
+
+    if (my $image = $c->image) {
+        if (! UNIVERSAL::isa($image, 'XML::RSS::LibXML::ElementSpec')) {
+            my $inode;
+
+            $inode = $dom->createElement('image');
+            $self->create_element_from_spec($image, $dom, $inode, \%ImageElements);
+            $self->create_extra_modules($image, $dom, $inode, $c->namespaces);
+            $channel->appendChild($inode);
+        }
+    }
+
+    if (my $textinput = $c->textinput) {
+        if (! UNIVERSAL::isa($textinput, 'XML::RSS::LibXML::ElementSpec')) {
+            my $inode;
+
+            $inode = $dom->createElement('textinput');
+            $self->create_element_from_spec($textinput, $dom, $inode, \%TextInputElements);
+            $self->create_extra_modules($textinput, $dom, $inode, $c->namespaces);
+            $channel->appendChild($inode);
+        }
+    }
+
+    foreach my $type qw(day hour) {
+        my $field = 'skip' . ucfirst($type) . 's';
+        my $skip = $c->$field;
+        if ($skip && defined $skip->{$type}) {
+            my $sd = $dom->createElement($field);
+            my $d  = $dom->createElement($type);
+            $d->appendChild($dom->createTextNode($skip->{$type}));
+            $sd->appendChild($d);
+            $channel->appendChild($sd);
+        }
+    }
     $root->appendChild($channel);
 }
 
@@ -241,7 +269,6 @@ sub create_items
     foreach my $i ($c->items) {
         my $item = $dom->createElement('item');
         $self->create_element_from_spec($i, $dom, $item, \%ItemElements);
-        $self->create_extra_modules($i, $dom, $item, $c->namespaces);
 
         $channel->appendChild($item);
     }
